/**
 * Created by 赵俊明 Z.JM on 2015/10/10.
 * 本校验指令集合内聚angularJs内置格式化指令，扩展时注意保证其服务函数完全闭合，不污染全局变量。
 */

angular.module("pf.directives.modelFormatDirective", [], ["$compileProvider", function ($compileProvider) {

        var smallKeyBoard = function (event) {
            var which = event.which;
            return (which >= 96 && which <= 105);
        };

        var numberKeyBpoard = function (event) {
            var which = event.which;
            return (which >= 48 && which <= 57) && !event.shiftKey;
        };

        var functionKeyBoard = function (event) {
            var which = event.which;
            return (which <= 40) || (navigator.platform.indexOf("Mac") > -1 && event.metaKey) || (navigator.platform.indexOf("Win") > -1 && event.ctrlKey);
        };

        var currencyKeyBoard = function (event, viewValue) {
            var which = event.which;
            return (viewValue.toString().indexOf('￥') === -1 && which === 52 && event.shiftKey);
        };

        var floatKeyBoard = function (event, viewValue) {
            var which = event.which;
            return [188].indexOf(which) != -1 || (which === 190 || which === 110) && viewValue.toString().indexOf('.') === -1;
        };

        var modelFormatConfig = {
            currency: {
                formatter: function (args) {
                    var modelValue = args.$modelValue,
                        filter = args.$filter,
                        attrs = args.$attrs,
                        $eval = args.$eval;

                    var val = filter("currency")(modelValue);
                    return attrs.prefixed && $eval(attrs.prefixed) ? val : (val ? val.substr(1) : "");
                },
                parser: function (args) {
                    var viewValue = args.$viewValue;
                    var num = viewValue ? viewValue.toString().replace(/[^0-9.]/g, '') : 0;
                    var result = parseFloat(num, 10);
                    return isNaN(result) ? undefined : parseFloat(result.toFixed(2));
                },
                isEmpty: function (value) {
                    return !value.$modelValue;
                },
                keyDown: function (args) {
                    var event = args.$event,
                        viewValue = args.$viewValue,
                        modelValue = args.$modelValue;

                    if (!(smallKeyBoard(event) || numberKeyBpoard(event) || functionKeyBoard(event) || currencyKeyBoard(event, viewValue) || floatKeyBoard(event, viewValue))) {
                        event.stopPropagation();
                        event.preventDefault();
                    }


                    if (null != modelValue && modelValue.toString().split('.').length > 1 && modelValue.toString().split('.')[1].toString().length > 1 && event.which != 8) {

                        event.stopPropagation();
                        event.preventDefault();
                    }
                }
            },
            digit: {
                formatter: function (args) {
                    return args.$modelValue;
                },
                parser: function (args) {
                    return args.$viewValue ? args.$viewValue.toString().replace(/[^0-9]/g, '') : undefined;
                },
                isEmpty: function (value) {
                    return !value.$modelValue;
                },
                keyDown: function (args) {
                    var event = args.$event;

                    if (!(smallKeyBoard(event) || numberKeyBpoard(event) || functionKeyBoard(event))) {
                        event.stopPropagation();
                        event.preventDefault();
                    }
                }
            },
            int: {
                formatter: function (args) {
                    var modelValue = args.$modelValue,
                        filter = args.$filter;
                    return filter("number")(modelValue);
                },
                parser: function (args) {
                    var val = parseInt(args.$viewValue.toString().replace(/[^0-9]/g, ''), 10);
                    return isNaN(val) ? undefined : val;
                },
                isEmpty: function (value) {
                    return !value.$modelValue;
                },
                keyDown: function (args) {
                    var event = args.$event;

                    if (!(smallKeyBoard(event) || numberKeyBpoard(event) || functionKeyBoard(event))) {
                        event.stopPropagation();
                        event.preventDefault();
                    }
                }
            }
        };


        /** 表单输入格式化 **/
        $compileProvider.directive('modelFormat', ["$filter", "$parse",
            function ($filter, $parse) {
                return {
                    require: 'ngModel',
                    link: function (scope, element, attrs, ctrl) {
                        var config = modelFormatConfig[attrs.modelFormat] || {};

                        var parseFuction = function (funKey) {
                            if (attrs[funKey]) {
                                var func = $parse(attrs[funKey]);
                                return (function (args) {
                                    return func(scope, args);
                                });
                            }
                            return config[funKey];
                        };

                        var formatter = parseFuction("formatter");
                        var parser = parseFuction("parser");
                        var isEmpty = parseFuction("isEmpty");
                        var keyDown = parseFuction("keyDown");

                        var getModelValue = function () {
                            return $parse(attrs.ngModel)(scope);
                        };

                        if (keyDown) {
                            element
                                .bind("blur", function () {
                                    element.val(formatter({
                                        "$modelValue": getModelValue(),
                                        "$filter": $filter,
                                        "$attrs": attrs,
                                        "$eval": scope.$eval
                                    }));
                                })
                                .bind("keydown", function (event) {
                                    keyDown({
                                        "$event": event,
                                        "$viewValue": element.val(),
                                        "$modelValue": getModelValue(),
                                        "$attrs": attrs,
                                        "$eval": scope.$eval,
                                        "$ngModelCtrl": ctrl
                                    });
                                });
                        }


                        ctrl.$parsers.push(function (viewValue) {
                            return parser({
                                "$viewValue": viewValue,
                                "$attrs": attrs,
                                "$eval": scope.$eval
                            });
                        });

                        ctrl.$formatters.push(function (value) {
                            return formatter({
                                "$modelValue": value,
                                "$filter": $filter,
                                "$attrs": attrs,
                                "$eval": scope.$eval
                            });
                        });

                        ctrl.$isEmpty = function (value) {
                            return isEmpty({
                                "$modelValue": value,
                                "$attrs": attrs,
                                "$eval": scope.$eval
                            });
                        };
                    }
                };
            }
        ]);


    }])
    .directive("textTransformBankcard", [function () {
        return {
            restrict: "A",
            require: "?ngModel",
            link: function (scope, element, attrs, ngModelController) {
                if (!ngModelController) {
                    return;
                }

                var isValidate = true;
                /** 监听属性值改变 以应对动态的校验规则 **/
                attrs.$observe("textTransformBankcard", function (value) {
                    isValidate = scope.$eval(value);
                    if (null == value || '' == value) {
                        isValidate = true;
                    }
                });

                ngModelController.$viewChangeListeners.push(function () {
                    if (!ngModelController.$isEmpty(ngModelController.$viewValue)) {
                        var viewValue = ngModelController.$viewValue.toString().replace(/(^\s*)|(\s*$)/g, "");
                        viewValue=viewValue.length>23?viewValue.substring(0,23):viewValue;
                        viewValue = viewValue.toString().replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, "$1 ");
                        ngModelController.$setViewValue(viewValue);
                        ngModelController.$render();
                    }
                });

                ngModelController.$formatters.push(function (modelValue) {
                    if (modelValue) {
                        modelValue = modelValue.toString().replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, "$1 ");
                    }
                    return modelValue;
                });

                ngModelController.$parsers.push(function (viewValue) {
                    if (viewValue) {
                        viewValue = viewValue.toString().replace(/\s+/g, '');
                    }
                    return viewValue;
                });
            }
        }

    }]);








