/**
 * Created by Z.JM on 2015/12/14.
 */
"use strict";
angular.module('pfApp')
    .directive('iframeLoadComplete', ['$ionicModal', '$sce', 'RestFulURLConstants', 'UserInfoService', '$timeout', function ($ionicModal, $sce, RestFulURLConstants, UserInfoService, $timeout) {

        var directiveDefinitionObject = {
            restrict: "EA",
            scope: {
                iframeLoadComplete: "&"
            },
            link: function ($scope, $element, $attrs) {
                $element.on("load", function (e) {
                    $timeout(function () {
                        $scope.iframeLoadComplete($element);
                    });
                });
            }
        };


        return directiveDefinitionObject;
    }])
    .factory("FinanceAgreementCodeTrustURL", ['$sce', 'RestFulURLConstants', 'UserInfoService', 'HttpParseParameters', 'SHA1ChannelCryptograph',
        function ($sce, RestFulURLConstants, UserInfoService, HttpParseParameters, SHA1ChannelCryptograph) {
            function getAgreemenSrcByType(agreementCode) {
                //url
                if (/^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i.test(agreementCode)) {
                    return $sce.trustAsResourceUrl(agreementCode);
                }
                //code
                return $sce.trustAsResourceUrl(RestFulURLConstants.unitResource.salesAgreement + "/" + agreementCode + "?tokenId=" + UserInfoService.getUserTokenId() + HttpParseParameters.parse(SHA1ChannelCryptograph()));
            };

            return getAgreemenSrcByType;
        }])
    .factory("FinanceAgreementService", ['$rootScope', '$ionicModal', '$interval', 'FinanceAgreementCodeTrustURL', '$state', '$ionicHistory',
        function ($rootScope, $ionicModal, $interval, FinanceAgreementCodeTrustURL, $state, $ionicHistory) {

            return function ($parentScope) {

                var $scope = ($parentScope || $rootScope).$new();

                function show() {

                    if ($scope.viewType == "view") {
                        $state.go("showFinanceAgreement", {
                            title: $scope.title,
                            code: $scope.code,
                            viewType: $scope.viewType
                        });
                    } else {
                        $scope.agreementSrc = FinanceAgreementCodeTrustURL($scope.code);
                        $scope.modal = null;

                        $scope.iframeLoadComplete = function (e) {
                            $scope.iframeProcess = 100;
                        };

                        $scope.closeAgreementModal = function () {
                            $scope.modal && $scope.modal.hide();
                        };

                        $scope.$on('$destroy', function () {
                            if ($scope.modal) {
                                if ($scope.processInterval) {
                                    $interval.cancel($scope.processInterval);
                                }
                                $scope.modal.remove();
                            }
                        });

                        $scope.$on('modal.hidden', function () {
                            $scope.agreementClose($scope);
                        });

                        var templateUrl = 'directives/privateOriginal/template/financeAgreement/agreementHTMLModal.html';


                        $ionicModal.fromTemplateUrl(templateUrl, {
                            scope: $scope,
                            animation: 'slide-in-right'
                        }).then(function (modal) {
                            $scope.modal = modal;
                            $scope.modal.show();

                            $scope.iframeProcess = 0;

                            $scope.processInterval = $interval(function () {
                                if ($scope.iframeProcess < 100) {
                                    if (parseFloat($scope.iframeProcess) < 60) {
                                        $scope.iframeProcess = $scope.iframeProcess + 1;
                                    }
                                    if (parseFloat($scope.iframeProcess) < 70) {
                                        $scope.iframeProcess = $scope.iframeProcess + 0.5;
                                    }
                                    if (parseFloat($scope.iframeProcess) < 80) {
                                        $scope.iframeProcess = $scope.iframeProcess + 0.25;
                                    }
                                    if (parseFloat($scope.iframeProcess) < 90) {
                                        $scope.iframeProcess = $scope.iframeProcess + 0.125;
                                    }
                                    if (parseFloat($scope.iframeProcess) < 95) {
                                        $scope.iframeProcess = $scope.iframeProcess + 0.075;
                                    }
                                    if (parseFloat($scope.iframeProcess) < 99) {
                                        $scope.iframeProcess = $scope.iframeProcess + 0.0345;
                                    }
                                } else {
                                    if ($scope.processInterval) {
                                        $interval.cancel($scope.processInterval);
                                    }
                                }
                            }, 20);
                        });
                    }

                };

                function hide() {
                    if ($scope.viewType == "view") {
                        $ionicHistory.goBack();
                    } else {
                        $scope.closeAgreementModal();
                    }
                }

                return {
                    show: show,
                    hide: hide
                };

            };

        }])
    .directive('financeAgreement', ['FinanceAgreementService', '$state',
        function (FinanceAgreementService, $state) {

            var directiveDefinitionObject = {
                restrict: "EA",
                scope: {
                    title: "@",
                    code: "@",
                    agreementClose: "&",
                    viewType: "@"
                },
                compile: function compile(tElement, tAttrs, transclude) {

                    return function ($scope, element, attrs) {
                        element.on("click ", function (ev) {
                            ev.stopPropagation();
                            $scope.viewAgreement();
                        });

                        $scope.viewAgreement = function () {
                            FinanceAgreementService($scope).show();
                        };
                    }
                }
            };

            return directiveDefinitionObject;
        }])
    .directive('countDown', ['$rootScope', '$interval', function ($rootScope, $interval) {
        var directiveDefinitionObject = {
            restrict: "A",
            scope: {
                intervalTime: "@",
                step: "@",
                counter: '='
            },
            compile: function compile(tElement, tAttrs, transclude) {
                //link
                return function (scope, element, attrs) {
                    var countDownInterval = $interval(function () {
                        if (angular.isNumber(scope.counter)) {
                            if (scope.counter > 0) {
                                scope.counter -= scope.step || 1000;
                            }
                        } else if (angular.isArray(scope.counter)) {
                            for (var i = 0, n = scope.counter.length; i < n; i++) {
                                if (scope.counter[i] > 0) {
                                    scope.counter[i] -= scope.step || 1000;
                                }
                            }
                        }
                    }, scope.intervalTime || 1000);

                    //scope.countDownInterval.flush(scope.intervalTime || 1000);

                    scope.$on("$destroy", function () {
                        $interval.cancel(countDownInterval);
                    });
                };
            }
        };
        return directiveDefinitionObject;
    }])
    .directive('remoteImage', ['$sce', 'RestFulURLConstants', 'UserInfoService', function ($sce, RestFulURLConstants, UserInfoService) {
        var directiveDefinitionObject = {
            restrict: "A",
            priority: 10,
            scope: {
                ngSrc: "@"
            },
            compile: function compile(tElement, tAttrs, transclude) {
                //link
                return function (scope, element, attrs) {
                    if (!/^.*[^a][^b][^c]\.(?:png|jpg|bmp|gif|jpeg)$/.test(scope.ngSrc) && !/^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i.test(scope.ngSrc)) {
                        scope.ngSrc = $sce.trustAsResourceUrl(RestFulURLConstants.unitResource.downImageByFileOid + "/" + scope.ngSrc + "?tokenId=" + UserInfoService.getUserTokenId());
                    }
                };
            }
        };
        return directiveDefinitionObject;
    }])
    .directive("ngModelClear", ['$timeout', function ($timeout) {
        return {
            restrict: "A",
            scope: {
                ngModelClear: "=ngModelClear"
            },
            compile: function compile(tElement, tAttrs) {

                angular.element(tElement).addClass("ng-hide");

                return function (scope, element, attrs) {

                    scope.$watch("ngModelClear", function () {

                        scope.ngModelClear.$isEmpty(scope.ngModelClear.$viewValue) ? element.addClass("ng-hide") : element.removeClass("ng-hide");

                        scope.ngModelClear.$parsers.push(function (viewValue) {
                            scope.ngModelClear.$isEmpty(scope.ngModelClear.$viewValue) ? element.addClass("ng-hide") : element.removeClass("ng-hide");
                            return viewValue;
                        });

                        element.bind('click', function (e) {
                            $timeout(function () {
                                scope.ngModelClear.$setViewValue(null, null);
                                scope.ngModelClear.$render();
                            });
                        });
                    });

                }
            }
        };
    }])
  .directive('cgImg', ['$timeout', function($timeout) {
    return {
      restrict: 'E',
      replace: true,
      template: '<img ng-src="{{src}}">',
      link: function(scope, element, attrs) {
        var time = new Date().getTime();
        var url = attrs.path;
        var fileName = attrs.oid;
        var isSetSize = attrs.size;

        if(navigator.download && navigator.download.start) {
          navigator.download.start({
            success: function(path) {
              scope.src = path;
            },
            error: function(error) {
              scope.src = url;
            },
            mills:{'url':encodeURIComponent(url),'type': time,'l':'1','fileName':fileName, 'cache': true}
          });
        } else {
          scope.src = url;
        }

        element[0].onload = function() {
          if(isSetSize) {
            element[0].width = element[0].width/2;
            element[0].style.display = 'inline';
          }
        };

      }
    }
  }])

    //图片校验码指令 20160829 created by yanliang
    .directive('picVerificationCode',['$interval','UserInfoService','RestFulURLConstants','$cordovaToast', 'HttpParseParameters','SHA1ChannelCryptograph', '$rootScope',function ($interval,UserInfoService,RestFulURLConstants,$cordovaToast, HttpParseParameters, SHA1ChannelCryptograph, $rootScope) {
        return {
            restrict:'EA',
            replace:true,
            template:"<button data-tap-disabled=\"true\" class=\"button button-clear buttonz-no-padding\" on-tap=\"resetPicVerCode()\">" +
            "<img ng-if=\"picVerUrl\" ng-src=\"{{picVerUrl}}\">" +
            "</button>",
            scope:{
                params:'=',
                actions:'='
            },
            link:function ($scope, $iElement, $iAttrs, $controller, $transcludeFn) {

                var userAccountKey = $iElement.key || 'userAccount';
                var params = $scope.params;
                if (!params) {
                    params = {};
                    params[userAccountKey] = UserInfoService.getUserInfo().cellPhone;
                }

                //消息提示
                var uaErrorMsg = $iAttrs.shortMsg ? ('请先输入正确的'+$iAttrs.shortMsg) :
                    $iAttrs.fullMsg ? $iAttrs.fullMsg : '请先输入正确的手机号';

                //两次换图之间的间隔时间
                var intervalSecond = 3;
                var lastChangePictTime = null;
                //点击验证图切换过程中的等待时间
                var picCodeWaitCount = 0;
                //获取图片验证码
                ($scope.setPicVerUrl = function (date) {
                    //tokenId,HttpParseParameters服务，是从remoteImage filter中取得，
                    // 因为此filter中加上'?'，应该去修改过滤器，现在展示但是将参数放在
                    //$rootScope.appFeatures._UUID
                    $scope.picVerUrl = RestFulURLConstants.publicService.getPicVerificationCode+
                        "?userAccount="+$rootScope.appFeatures._UUID+
                        "&timestamp="+date+
                        "&tokenId=" + UserInfoService.getUserTokenId()+
                        HttpParseParameters.parse(SHA1ChannelCryptograph());

                    //更换图形验证码成功后，执行操作
                    if($scope.actions && $scope.actions.changePic)
                        $scope.actions.changePic();
                })(new Date().getTime());
                //重置图片验证码
                $scope.resetPicVerCode = function () {
                    if (!params[userAccountKey]) {
                        // $cordovaToast.show(uaErrorMsg, 'short', 'center');
                        // return;
                    }

                    var nowTime = new Date().getTime();
                    //如果已处于等待换图计时，或者两次换图时间少于2秒，则返回
                    if (picCodeWaitCount > 0) {
                        // console.log(nowTime-lastChangePictTime+'切换验证图频繁，请稍后重试！');
                        $cordovaToast.hide();
                        $cordovaToast.show('请勿频繁切换验证图', 'short', 'center');
                        return;
                    }
                    if (lastChangePictTime && (nowTime-lastChangePictTime < 2000)) {
                        // console.log(nowTime-lastChangePictTime+'切换验证图频繁，请稍后重试！');
                        $cordovaToast.hide();
                        $cordovaToast.show('请勿频繁切换验证图', 'short', 'center');
                        countInterval();
                        return;
                    }
                    //允许切换验证图，记录当前时间
                    lastChangePictTime = nowTime;
                    $scope.setPicVerUrl(new Date().getTime());
                };

                //切换验证图过于频繁的倒计时
                var countInterval = function () {
                    picCodeWaitCount = intervalSecond;
                    $interval(function () {
                        picCodeWaitCount--;
                    },1000,intervalSecond);
                };

                if ($scope.actions)
                    $scope.actions.resetPicVerCode = $scope.resetPicVerCode;

            }
        }
    }])

    //手机校验码指令 20160831 created by yanliang
    //changelog: 20160918 使用手机验证码指令时的倒计状态都会被缓存在$rootScope中，可查看revision 118140，看变化的代码有哪些.
    .directive('phoneVerificationCode',['$rootScope', 'SendValidateCodeService','MobileValidateCodeTypeConstants', '$state',function ($rootScope, SendValidateCodeService, MobileValidateCodeTypeConstants, $state) {
        return {
            restrict:'EA',
            replace:true,
            templateUrl:'directives/privateOriginal/template/phoneVerificationCode.html',
            scope:{
                phonenumber:'=',
                action:'='
            },
            link:function ($scope, $iElement, $iAttrs, $controller, $transcludeFn) {
                //example:<phone-verification-code phonenumber="confirmModel.newPhone" code-type="CHANGE_BANK_MOBILE"></phone-verification-code>
                var codeType = MobileValidateCodeTypeConstants[$iAttrs.codeType || 'REGISTER'];

                //初始化验证参数对象时，从缓存中取出倒计时缓存。
                var currentCachedKey = $state.current.name + $iAttrs.id;
                var cachedPhoneCodeObj = $rootScope.cachedPhoneCodeObj[currentCachedKey];
                if(cachedPhoneCodeObj){
                    $scope.phoneCodeObj = cachedPhoneCodeObj;
                }else {
                    $scope.phoneCodeObj = {};
                    $rootScope.cachedPhoneCodeObj[currentCachedKey] = $scope.phoneCodeObj;
                }
                // $scope.phoneCodeObj = cachedPhoneCodeObj || {};

                $scope.shortMessageValidateCode = function () {
                    //因为之前封装的服务是直接在$scope上取$scope.user.cellphone为userAccount，兼容以前的写法
                    $scope.phoneCodeObj.user = {
                        cellPhone:$scope.phonenumber
                    };
                    SendValidateCodeService.sendValidateCode($scope.phoneCodeObj, codeType);
                };

                // 销毁指令时，将倒计时缓存下来。
                // $scope.$on("$destroy", function (event) {
                //     //缓存验证码对象
                //     $rootScope.cachedPhoneCodeObj[currentCachedKey] = $scope.phoneCodeObj;
                // });

                //方法初始化
                if($scope.action){

                    //绑定发送验证码方法，外部可使用
                    $scope.action.sendCode = $scope.shortMessageValidateCode;
                    //强制充值送验证码
                    $scope.action.forceSendCode = function (phonenum) {
                        if(phonenum)
                            $scope.phonenumber = phonenum;
                        $scope.phoneCodeObj = {};
                        $scope.shortMessageValidateCode();
                    };
                    if($scope.action.initAction)
                        $scope.action.initAction();
                }
            }
        }
    }])

    //选中input框后 父元素增加focus class 20160901 created by yanliang
    .directive('focusEmphasis',[function () {
        return {
            restrict:'CA',
            link:function ($scope, $iElement, $iAttrs, $controller, $transcludeFn) {
                // var jqEle = angular.element($iElement[0]);
                // var parentJqEle = angular.element($iElement[0]).parent();
                // jqEle.bind('onfocus',function (e) {
                //     parentJqEle.addClass('focus');
                // });
                // jqEle.bind('onblur',function (e) {
                //     parentJqEle.removeClass('focus');
                // });

                //!上面这种使用jq的方法不起作用，因为没引库？使用最原始的方法
                var ele = $iElement[0];
                var parentEle = angular.element(ele).parent()[0];
                ele.onfocus = function(){
                    parentEle.classList.add('focus');
                };
                ele.onblur = function () {
                    parentEle.classList.remove('focus');
                };

            }
        }
    }]);
