/**
 * Created by Z.JM on 2015/11/30.
 */
;
'use strict';
angular.module('pf.directives.formatterFilter', [])
    .filter('RMBUppercase', [function () {

        var CN_NUMBERS = ["零", "壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖"]; // 汉字的数字
        var CN_INT_RADICE = ["", "拾", "佰", "仟"]; // 基本单位
        var CN_INT_UNITS = ["", "万", "亿", "兆"]; // 对应整数部分扩展单位
        var CN_DEC_UNITS = ["角", "分", "毫", "厘"]; // 对应小数部分单位
        var CN_INTEGER = "整"; // 整数金额时后面跟的字符
        var CN_INT_LAST = "元"; // 整型完以后的单位
        var MAX__formatMoney = 999999999999999.9999; // 最大处理的数字

        return function (_formatMoney) {
            if (_formatMoney === "" || isNaN(_formatMoney) || !!!_formatMoney) {
                return "";
            }

            _formatMoney = parseFloat(_formatMoney);

            if (_formatMoney >= MAX__formatMoney) {
                //超过最大转换限制
                return "";
            }

            var _IntegerNum = null; // 金额整数部分
            var _DecimalNum = null; // 金额小数部分
            var _ChineseStr = ""; // 输出的中文金额字符串
            var _Parts = null; // 分离金额后用的数组，预定义

            if (_formatMoney == 0) {
                _ChineseStr = CN_NUMBERS[0] + CN_INT_LAST + CN_INTEGER;
                return _ChineseStr;
            }

            _formatMoney = _formatMoney.toString(); // 转换为字符串
            if (_formatMoney.indexOf(".") == -1) {
                _IntegerNum = _formatMoney;
                _DecimalNum = '';
            } else {
                _Parts = _formatMoney.split(".");
                _IntegerNum = _Parts[0];
                _DecimalNum = _Parts[1].substr(0, 4);
            }

            if (parseInt(_IntegerNum, 10) > 0) {// 获取整型部分转换
                var zeroCount = 0;
                for (var i = 0, _IntegerNumLength = _IntegerNum.length; i < _IntegerNumLength; i++) {
                    var n = _IntegerNum.substr(i, 1);
                    var p = _IntegerNumLength - i - 1;
                    var q = p / 4;
                    var m = p % 4;
                    if (n == "0") {
                        zeroCount++;
                    } else {
                        if (zeroCount > 0) {
                            _ChineseStr += CN_NUMBERS[0];
                        }
                        zeroCount = 0; // 归零
                        _ChineseStr += CN_NUMBERS[parseInt(n)] + CN_INT_RADICE[m];
                    }
                    if (m == 0 && zeroCount < 4) {
                        _ChineseStr += CN_INT_UNITS[q];
                    }
                }
                _ChineseStr += CN_INT_LAST;
                // 整型部分处理完毕
            }
            if (_DecimalNum != '') {// 小数部分
                for (var i = 0, _DecimalNumLength = _DecimalNum.length; i < _DecimalNumLength; i++) {
                    var n = _DecimalNum.substr(i, 1);
                    if (n != '0') {
                        _ChineseStr += CN_NUMBERS[Number(n)] + CN_DEC_UNITS[i];
                    }
                }
            }
            if (_ChineseStr == '') {
                _ChineseStr += CN_NUMBERS[0] + CN_INT_LAST + CN_INTEGER;
            } else if (_DecimalNum == '') {
                _ChineseStr += CN_INTEGER;
            }
            return _ChineseStr;

        };
    }])
    .filter('TimerMillisecondFormat', [function () {

        return function (timerMillisecond, format) {

            function day(time) {
                return parseInt(time / (1000 * 60 * 60 * 24));
            };
            function hour(time) {
                var result = parseInt((time / (1000 * 60 * 60)) % 24);
                return result > 9 ? result : '0' + result;
            };
            function minute(time) {
                var result = parseInt((time / (1000 * 60)) % 60);
                return result > 9 ? result : '0' + result;
            };
            function second(time) {
                var result = parseInt((time / 1000) % 60);
                return result > 9 ? result : '0' + result;
            };
            function milliSecond(time) {
                return time % 1000;
            };

            var countDownFormat = {
                day: 0,
                hour: 0,
                minute: 0,
                second: 0,
                milliSecond: 0
            };

            if ('d' === format || 'D' == format) {
                countDownFormat = day(timerMillisecond);
            } else if ('h' === format || 'H' == format) {
                countDownFormat = hour(timerMillisecond);
            }
            else if ('m' === format || 'M' == format) {
                countDownFormat = minute(timerMillisecond);
            }
            else if ('s' === format || 'S' == format) {
                countDownFormat = second(timerMillisecond);
            }
            else if ('ms' === format || 'MS' == format) {
                countDownFormat = milliSecond(timerMillisecond);
            }
            else {
                countDownFormat = {
                    day: day(timerMillisecond),
                    hour: hour(timerMillisecond),
                    minute: minute(timerMillisecond),
                    second: second(timerMillisecond),
                    milliSecond: milliSecond(timerMillisecond)
                };
            }
            return countDownFormat;
        };

    }])
    .filter('remoteImage', ['$sce', 'RestFulURLConstants', 'UserInfoService', 'SHA1ChannelCryptograph', 'HttpParseParameters',
        function ($sce, RestFulURLConstants, UserInfoService, SHA1ChannelCryptograph, HttpParseParameters) {

            return function (src, frmatter) {

                if (/^.*[^a][^b][^c]\.(?:png|jpg|bmp|gif|jpeg)$/.test(src) || /^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i.test(src)) {
                    return $sce.trustAsResourceUrl(src);
                }

                return $sce.trustAsResourceUrl(RestFulURLConstants.unitResource.downImageByFileOid + "/" + src + "?tokenId=" + UserInfoService.getUserTokenId() + HttpParseParameters.parse(SHA1ChannelCryptograph()));

            };

        }])
    .filter('numberSplit', [function () {

        return function (content, index) {
            return content ? content.replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, "$1 ") : content;
        }
    }])
    .filter('phoneHide', function() {
        return function(content) {
            return content ? content.replace(/\s/g, '').replace(/(\d{3})(\d{4})(\d{4})/g, "$1 **** $3") : content;
        }
    })
    .filter('monthAndDaySplit', [function() {
        return function(content) {
            return content?content.replace(/\d{4}(\d{2})(\d{2})$/, '$1'+'-'+'$2'):content;
        }
    }]);
