// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'

/** 全局lib库 */
//'ngIOS9UIWebViewPatch' ios需要注入
var globalLibs = angular.module('pf.globalLibs', ['ngCordova', 'oc.lazyLoad', 'webStorageModule', 'ngImgCrop', 'monospaced.qrcode', 'radialIndicator']);
/* 全局服务库*/
var globalService = angular.module('pf.globalService', ['pf.services.httpResourceService', 'pf.services.storage', 'ms.gestureLock', 'ms.featureGuide', 'pf.services.sendNotificationMessage']);
/* 全局指令库*/
var globalDirectives = angular.module('pf.globalDirectives', ['pf.directives.validateDirective', 'pf.directives.modelFormatDirective']);
/* 全局过滤器库*/
var globalFilters = angular.module('pf.globalFilters', ['pf.directives.formatterFilter']);

var pfApp = angular.module('pfApp', ['ionic', 'pfApp.constants', 'pf.globalLibs', 'pf.globalService', 'pf.globalDirectives', 'pf.globalFilters'])

    //初始化全局参数
    .run(['$rootScope',
        function ($rootScope) {

            //是否是安卓系统 boolean
            $rootScope.isAndroid = ionic.Platform.isAndroid();

            //是否登录 boolean
            $rootScope.isGlobalLogin = false;

            //session超时时间 number
            $rootScope._SESSION_TIME_OUT_SECOND = 0;

            //session是否超时 boolean
            $rootScope.IS_SESSION_TIME_OUT = false;

            //网络在线
            $rootScope.isOnline = true;

            //APP特性
            $rootScope.appFeatures = {};

            //是否是安全通道
            $rootScope.IsSecurityHTTPRequestChannel = false;

            //手机验证码缓存对象
            $rootScope.cachedPhoneCodeObj = {};

        }])
    //监听网络连接，根据机器配置选择渲染模式
    .run(['$rootScope', '$cordovaNetwork', '$ionicPlatform', 'DevelopmentModeConstants', '$cordovaAppVersion',
        function ($rootScope, $cordovaNetwork, $ionicPlatform, DevelopmentModeConstants, $cordovaAppVersion) {
            $ionicPlatform.ready(function () {
                $rootScope.isOnline = $cordovaNetwork.isOnline();

                // listen for Online event
                $rootScope.$on('$cordovaNetwork:online', function (event, networkState) {
                    $rootScope.isOnline = true;
                });

                // listen for Offline event
                $rootScope.$on('$cordovaNetwork:offline', function (event, networkState) {
                    $rootScope.isOnline = false || DevelopmentModeConstants.debug;
                });

                $rootScope.appFeatures._platform = ionic.Platform.platform();
                $rootScope.appFeatures._platformVersion = ionic.Platform.version();
                var _device = ionic.Platform.device();
                $rootScope.appFeatures._UUID = _device.uuid;
                $rootScope.appFeatures._IMEI = _device.imei;

                $rootScope.appFeatures._APP_VERSION = "0.0.0";

                $cordovaAppVersion.getVersionNumber().then(function (version) {
                    $rootScope.appFeatures._APP_VERSION = version || $rootScope.appFeatures._APP_VERSION;
                });
            });


            if (ionic.Platform.isAndroid()) {
                if (ionic.Platform.version() > "4.4") {
                    ionic.Platform.setGrade('a');
                }
                else if (ionic.Platform.version() > "4.2") {
                    ionic.Platform.setGrade('b');
                } else {
                    ionic.Platform.setGrade('c');
                }
            }
        }])
    //全局设定
    .run(['$ionicPlatform', '$rootScope',
        function ($ionicPlatform, $rootScope) {
            $ionicPlatform.ready(function () {
                // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
                // for form inputs)
                if (window.cordova && window.cordova.plugins.Keyboard) {
                    cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                    cordova.plugins.Keyboard.disableScroll(true);
                }
                if (window.StatusBar) {
                    // org.apache.cordova.statusbar required
                    //StatusBar.styleDefault();
                    StatusBar.backgroundColorByHexString('#f96c41');
                }
            });
        }])
    //理由级别限制
    .run(['$ionicPlatform', '$rootScope', '$state', '$location', '$timeout', '$ionicHistory', 'UserInfoService', 'RouterLevel', '$ionicPopup',
        function ($ionicPlatform, $rootScope, $state, $location, $timeout, $ionicHistory, UserInfoService, RouterLevel, $ionicPopup) {

            //权限路由限制
            $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
                //根据排除等级的路由注册表，区分哪些需要登录，哪些需要实名认证，哪些是游客，从UserinfoService中可以获取用户权限级别
                // 游客级别：1;  登录级别：2; 实名级别：4;  根据用户级别进行评级，总值7(类似Linux权限策略)，根据评级匹配路由表
                //通过给$stateProvider.state设置data:{$stateLevel:4},在此可通过toState.data.$stateLevel匹配路由表，默认为1，游客访问级别
                //具体实现请开发人员及时更新路由表级别参数
                var urlPowerLevel = toState.data && toState.data.$stateLevel ? toState.data.$stateLevel : RouterLevel.AUTHEN_USER;
                var statePowerBack = (toState.data && toState.data.$statePowerBack) ? toState.data.$statePowerBack : false;

                var powerLevel = UserInfoService.getPowerLevel();
                if (powerLevel < urlPowerLevel) {
                    event.preventDefault();
                    if (powerLevel < RouterLevel.LOGIN_USER) {
                      if(toState.name == 'personCenter.messageTextDetail') {
                        $state.go('security.login', {toStateObj: {
                          url: toState.name,
                          params: toParams
                        }});
                      } else {
                        $state.go('security.login');
                      }

                    } else if (powerLevel < RouterLevel.AUTHEN_USER) {
                        $ionicPopup.confirm({
                            title: '温馨提示',
                            template: '你还没有实名认证，去实名认证？',
                            cancelText: '取消',
                            okText: '确定',
                            okType: 'button-panda'
                        }).then(function (res) {
                            if (res) {
                                $state.go('personCenter.identification');
                            } else {
                                statePowerBack && $ionicHistory.backView() && $ionicHistory.goBack();
                            }
                        });
                    } else {
                        $state.go('security.login');
                    }
                }
            });

        }])
    //手势登录，session延长时间
    .run(['$ionicPlatform', '$rootScope', '$state', '$timeout', '$ionicHistory', 'UserInfoService', '$ionicPopup', 'localStorageServices', '$PFHttpResource', 'RestFulURLConstants', 'WebStorageConstants', 'GestureService', 'FeatureGuideService', 'MSGManage', '$FeatureGuide', '$gestureLock', 'GestureLockConstants', 'ChannelConstants',
        function ($ionicPlatform, $rootScope, $state, $timeout, $ionicHistory, UserInfoService, $ionicPopup, localStorageServices, $PFHttpResource, RestFulURLConstants, WebStorageConstants, GestureService, FeatureGuideService, MSGManage, $FeatureGuide, $gestureLock, GestureLockConstants, ChannelConstants) {

            //$rootScope.IS_SESSION_TIME_OUT = true;

            function hideSplashScreen() {
                $timeout(function () {
                    navigator.splashscreen && navigator.splashscreen.hide();
                }, 500);//IOS,100,android:500
            };

            function gestureLockControllerChain() {

                //如果设置了手势
                if (GestureService.isSetGesture()) {
                    if (!$gestureLock.isExits($gestureLock.GESTURE_LOCK_TYPE.VERIFICATION)) {
                        if (GestureService.getVerificationTimes() > 0) {
                            $gestureLock.verification(GestureService.getGesture(), GestureService.verificationFunction, "./img/svg/icon.svg", GestureLockConstants)
                                .then(function (res) {
                                    GestureService.setVerificationTimes(GestureLockConstants.verificationTimes);
                                    //解锁成功,发一个空请求，检测是否登录过期，如果过期则登录
                                    var userInfo = localStorageServices.getStorage(WebStorageConstants.CURRENT_USER_INFO);
                                    var user = {isAutoLogin: 'Y'};
                                    user.userAccount = userInfo.userAccount;
                                    user.userPassword = CryptoJS.MD5(userInfo.userPassword).toString();

                                    new $PFHttpResource(RestFulURLConstants.security.login, user)
                                        .then(function (data) {
                                            data.context.sysUserInfo.userPassword = userInfo.userPassword;
                                            UserInfoService.setUserInfo(data.context.sysUserInfo);
                                            UserInfoService.setUserTokenId(data.context.tokenId);
                                            UserInfoService.setElectronAcct(data.context.electronAcct);
                                            //连接消息中心
                                            try {
                                                MSGManage.wsconnect();
                                            } catch (e) {
                                            }
                                        }, function (error) {
                                            $state.go("tab.home", {goLogin: true});
                                        });
                                    hideSplashScreen();
                                }, function (error) {
                                    //其他账户登录
                                    if (error.status == $gestureLock.GESTURE_LOCK_STATUS.OTHER_ACCOUNT) {
                                        //退出账户，但不清除手势密码
                                        GestureService.removeGesture();
                                    } else if (error.status == $gestureLock.GESTURE_LOCK_STATUS.FORGET_PASSWORD) {
                                        //退出账户，并清除手势密码
                                        GestureService.removeGesture();
                                    } else if (error.status == $gestureLock.GESTURE_LOCK_STATUS.VERIFICATION_TIMES_OUT) {
                                        //退出账户，并清除手势密码
                                        GestureService.removeGesture();
                                    } else {
                                        //退出账户，并清除手势密码
                                        GestureService.removeGesture();
                                    }

                                    UserInfoService.removeUserInfo();
                                    $state.go("tab.home", {goLogin: true});
                                    hideSplashScreen();
                                }, function (notify) {
                                    if (notify.status == $gestureLock.GESTURE_LOCK_STATUS.OPENED) {
                                        hideSplashScreen();
                                    } else if (notify.status == $gestureLock.GESTURE_LOCK_STATUS.ERROR) {
                                        //在有效的次数内，输入错误,震动
                                        var times = GestureService.getVerificationTimes() - 1;
                                        GestureService.setVerificationTimes(times);
                                    }
                                });
                        } else {
                            hideSplashScreen();
                            GestureService.removeGesture();
                            UserInfoService.removeUserInfo();
                        }
                    } else {
                        hideSplashScreen();
                    }
                } else {
                    hideSplashScreen();
                }

            };
            function  linkToMessageDetail(data) {
              var res = {
                msg: {
                  oid: data.msgid,
                  msgtype: data.msgtype,
                  msgurl: data.msgurl
                },
                type: data.msgtype == 1? 'push': 'system'
              };
              if(res.msg.msgurl) {
                var externalUrl = res.msg.msgurl;
                //如果后管没有在url上加 http头（或https），则自动加上http
                /*var ref = window.open(externalUrl.indexOf('http') == 0 ? externalUrl : ('http://' + externalUrl), '_blank', 'location=yes');
                 ref.addEventListener('loaderror', function (event) {
                    $cordovaToast.show('打开外部页面失败', 'short', 'center');
                 });*/

                $state.go('tab.h5', {title: '动态消息',url: externalUrl});
              } else {
                if(data.msgtype == 1) {//不需要登录
                  $state.go('personCenter.messageTextDetailNotLogin', res);
                } else if(data.msgtype == 2) {//需要登录
                  $state.go('personCenter.messageTextDetail', res);
                }
              }

            }
            //ios消息推送注册
            function registerIosPushMessage() {
              if(ChannelConstants.id === 25) { //熊猫金融
                if(window.plugins&&window.plugins.pushNotification&&window.plugins.pushNotification.register) { //register
                  var apnSuccessfulRegistration = function(token) {
                    $rootScope.iosPushMessageDeviceId = token;
                  };
                  var apnFailedRegistration = function(error) {
                    $rootScope.iosPushMessageDeviceId = false;
                  };
                  window.plugins.pushNotification.register(apnSuccessfulRegistration,
                    apnFailedRegistration, {
                      "badge": "true",
                      "sound": "true",
                      "alert": "true",
                      "ecb": "pushCallbacks.onNotificationAPN" //tell PushPlugin to call onNotificationAPN of global pushCallbacks object
                    });
                }
                if(window.plugins&&window.plugins.pushNotification&&window.plugins.pushNotification.getMessage) { //get message
                  var apnSuccessfulGetMessage = function(data) {
                    if(data) {
                      if(data.foreground == 0) { //data.foreground == 0表示应用在后台时收到消息
                        linkToMessageDetail(data);
                      }
                    }
                  };
                  var apnFailedGetMessage = function(error) {
                  };
                  window.plugins.pushNotification.getMessage(apnSuccessfulGetMessage,apnFailedGetMessage);
                }
              }
            }

            //打开应用
            $ionicPlatform.ready(function () {
               registerIosPushMessage();
                if(!ChannelConstants.isShowGuidPage) {
                  hideSplashScreen();
                  return false;
                }
                //检查概要是否显示
                if (FeatureGuideService.isShowedFeatureGuide()) {
                    //判断手势作用连
                    //gestureLockControllerChain();
                    hideSplashScreen();
                    //显示启动广告页
                    $FeatureGuide.showStartAd()
                     .then(function(res) {
                     }, function(error) {
                     });
                } else {
                    $FeatureGuide.show(FeatureGuideService.getFeatureGuideImages())
                        .then(function (res) {
                            FeatureGuideService.setFeatureGuideStatus(false);
                            //gestureLockControllerChain();
                            //registerIosPushMessage();
                        }, function (error) {
                            //gestureLockControllerChain();
                        }, function (notify) {
                            //打开指引之后就隐藏
                            if ($FeatureGuide.FEATURE_GUIDE_STATUS.OPENED == notify.status) {
                                hideSplashScreen();
                            }
                        });
                }

            });

            $ionicPlatform.on("pause", function () {
              if (ChannelConstants.isShowGuidPage && GestureService.isSetGesture() && !$gestureLock.isExits($gestureLock.GESTURE_LOCK_TYPE.VERIFICATION)) {
                navigator.splashscreen && navigator.splashscreen.show();
              }
            });

            $ionicPlatform.on("resume", function () {
              if (ChannelConstants.isShowGuidPage && $gestureLock.isExits($gestureLock.GESTURE_LOCK_TYPE.VERIFICATION)) {
                hideSplashScreen();
              }

              if($state.current.name === "electronicBill.electronicBillProduct") {
                  $state.reload();
              }
              $ionicPlatform.ready(function() { //android点击推送消息
                if(ChannelConstants.id === 25) {
                  if(navigator.pushmessage && navigator.pushmessage.pushmsg) {
                    function readMsgSuccess(data) {
                      //data = {"msgid":"17623716823","msgtype":"1", "msgurl": "https://www.baidu.com/"}
                      if(data) {
                        data = JSON.parse(data);
                        linkToMessageDetail(data);
                      }
                    }
                    function readMsgError(error) {
                    }
                    navigator.pushmessage.pushmsg({
                      success: readMsgSuccess,
                      error: readMsgError,
                      mills: {}
                    });
                  }
                }
              });

            });

            $rootScope.$watch("IS_SESSION_TIME_OUT", function (IS_SESSION_TIME_OUT) {

            });

        }])
    //再摁一次退出
    .run(['$ionicPlatform', '$rootScope', '$state', '$timeout', '$ionicHistory', '$cordovaToast', 'IONIC_BACK_PRIORITY', 'UserInfoService',
        function ($ionicPlatform, $rootScope, $state, $timeout, $ionicHistory, $cordovaToast, IONIC_BACK_PRIORITY, UserInfoService) {

            $rootScope.hardwareBackButtonOnExitFlag = false;

            function onHardwareBackButton(e) {
                var _current_state_name = $state.current.name;
                var backView = $ionicHistory.backView();

                if (backView && !(_current_state_name === "tab.home" || _current_state_name === "tab.financeManagement" || _current_state_name === "tab.auctionList" || _current_state_name === "tab.self" || "security.gestureUnlock" === _current_state_name)) {
                    // there is a back view, go to it
                    backView.go();
                } else {
                    // there is no back view, so close the app instead
                    if ($rootScope.hardwareBackButtonOnExitFlag) {

                        if (ionic.Platform.isAndroid()) {
                            ionic.Platform.exitApp();
                        }
                        else {
                            ionic.Platform.exitApp();
                        }
                    }

                    $rootScope.hardwareBackButtonOnExitFlag = true;
                    $cordovaToast.showShortBottom('再按一次退出');

                    $timeout(function () {
                        $rootScope.hardwareBackButtonOnExitFlag = false;
                    }, 2000);
                }
                e.preventDefault();
                return false;
            }

            $ionicPlatform.registerBackButtonAction(
                onHardwareBackButton, IONIC_BACK_PRIORITY.view
            );
        }])
    //更新版本
    .run(['UpdateApkService', '$rootScope', '$ionicPlatform',
        function (UpdateApkService, $rootScope, $ionicPlatform) {

            $rootScope._isShowUpdate = false;
            $rootScope._isDownloading = false;

            $rootScope.initHttpRequest = true;

            $ionicPlatform.ready(function () {
                UpdateApkService.checkeApkUpdate();
            });
        }]);
