/**
 * Created by Z.JM on 2015/11/10.
 *
 * url:请求地址
 * parameters:{}（参数）
 * loadModal:{show:true,template:"<ion-spinner></ion-spinner>"}(缺省) 可以自定义模板。
 * $PFHttpResource(url,parameters,loadModal).then(function(data){},function(error){});
 *
 */
angular.module('pf.services.httpResourceService', ['pfApp'])
    //弱加密认证
    .factory("SHA1ChannelCryptograph", ['AppInformation', function (AppInformation) {
        var sha1ChannelCryptograph = function () {
            var time = new Date().getTime();
            return {
                appid: AppInformation.appId,
                signature: CryptoJS.SHA1(AppInformation.appId + "_" + AppInformation.appKey + "_" + time).toString(),
                time: time
            };
        };

        return sha1ChannelCryptograph;
    }])
    //httpObject转get字符串
    .factory("HttpParseParameters", [function () {
        var r20 = /%20/g, rbracket = /\[\]$/;

        var buildParams = function (prefix, obj, add) {
            var name;

            if (angular.isArray(obj)) {

                angular.forEach(obj, function (v, i) {
                    if (rbracket.test(prefix)) {

                        add(prefix, v);

                    } else {

                        buildParams(prefix + "[" + ( typeof v === "object" ? i : "" ) + "]", v, add);
                    }
                });

            } else if (angular.isObject(obj)) {

                for (name in obj) {
                    buildParams(prefix + "[" + name + "]", obj[name], add);
                }

            } else {

                add(prefix, obj);
            }
        }

        var parseParam = function (a) {
            var prefix,
                s = [],
                add = function (key, value) {

                    value = angular.isFunction(value) ? value() : ( value == null ? "" : value );
                    s[s.length] = encodeURIComponent(key) + "=" + encodeURIComponent(value);
                };
            //判断是否为数组
            if (angular.isArray(a)) {

                angular.forEach(a, function (value, key) {
                    add(key, value);
                });

            } else {

                for (prefix in a) {
                    buildParams(prefix, a[prefix], add);
                }
            }
            // 返回序列化信息
            return s.join("&").replace(r20, "+");
        };

        return {
            parse: parseParam
        }

    }])
    //认证请求
    .factory("$SSLUnilateralismAuthenticationHttpRequest", ['$q', '$timeout', 'SecurityHTTPRequestChannel', '$filter',
        function ($q, $timeout, SecurityHTTPRequestChannel, $filter) {

            function request(type) {
                return function (url, parameters, dataType) {

                    var _defer = $q.defer();

                    SecurityHTTPRequestChannel.postHTTPRequest(url, parameters)
                        .then(function (result) {
                            if ($filter('uppercase')(dataType || '') == 'JSON') {
                                try {
                                    _defer.resolve(JSON.parse(result || '{}'));
                                } catch (e) {
                                    _defer.reject("返回数据格式错误");
                                }
                            } else {
                                _defer.resolve(result);
                            }
                        }, function (error) {

                            if (typeof error === 'string') {
                                if (error == 'time out') {
                                    _defer.reject("连接超时");
                                } else if (error == 'shankhands failed') {
                                    SecurityHTTPRequestChannel.initHTTPRequestChannelAlert(error);
                                    _defer.reject("会话超时，请重新登录!");
                                } else {
                                    _defer.reject("网络不稳定，请重新再试!");
                                }
                            } else {
                                _defer.reject("网络不稳定，请重新再试!");
                            }

                        });

                    return _defer.promise;
                };
            };

            return {
                post: request("post"),
                get: request("get")
            };
        }])
    //同意请求
    .factory("$PFHttpResource", ['SHA1ChannelCryptograph', 'DevelopmentModeConstants', '$SSLUnilateralismAuthenticationHttpRequest', '$rootScope', '$q', '$http', '$ionicLoading', 'HttpParseParameters', 'UserInfoService', '$cordovaToast', '$ionicPopup', '$timeout', '$state', 'VersionRandomSerialNO', 'ChannelConstants',
        function (SHA1ChannelCryptograph, DevelopmentModeConstants, $SSLUnilateralismAuthenticationHttpRequest, $rootScope, $q, $http, $ionicLoading, HttpParseParameters, UserInfoService, $cordovaToast, $ionicPopup, $timeout, $state, VersionRandomSerialNO, ChannelConstants) {

            var ionicLoadingStackCounter = 0;

            //默认请求配置
            var _httpConfig = {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                    'appversion': VersionRandomSerialNO.substring(5),
                    "channelId": ChannelConstants.id
                },
                timeout: 60000,  //默认请求时间
                transformRequest: function (_data) {
                    return HttpParseParameters.parse(_data || {});   //序列化请求参数
                },
                sha1Channel: true
            };

            //默认记载显示配置
            var _loadingConfig = {
                show: true,
                template: '<ion-spinner class="spinner-panda"></ion-spinner><div class="text-center"><span>正在加载</span></div>'
            };

            var _toastConfig = {
                show: true,
                length: "short",
                position: "center"
            }

            var closeIonicLoading = function (loadingConfig_) {
                $timeout(function () {
                    if (loadingConfig_.show) {
                        if (ionicLoadingStackCounter > 0) {
                            ionicLoadingStackCounter--;
                        }
                        if (ionicLoadingStackCounter == 0) {
                            $ionicLoading.hide();
                        }
                    }
                });
            };

            var showIonicLoading = function (loadingConfig_) {
                $timeout(function () {
                    if (loadingConfig_.show) {
                        if (ionicLoadingStackCounter == 0) {
                            $ionicLoading.show(loadingConfig_);
                        }
                        ionicLoadingStackCounter++;
                    }
                });
            };


            var SSLUnilateralismAuthenticationHttpRequest = function (_q, url, data, httpConfig_, loadingConfig_, toastConfig_) {
                showIonicLoading(loadingConfig_);
                $SSLUnilateralismAuthenticationHttpRequest.post(url, data, "json")
                    .then(function (data) {
                        closeIonicLoading(loadingConfig_);

                        if (data && data.code == "00") {
                            _q.resolve(data);

                        } else if(data && data.code == "02") { //版本升级
                            $cordovaToast.show(data.message || '当前版本过低，请升级！', toastConfig_.length, toastConfig_.position);
                            _q.reject();
                        } else if (data && data.code == "TIMEOUT") {
                            UserInfoService.removeUserInfo();
                            try {
                                $cordovaToast.show('登录已超时，请重新登录', toastConfig_.length, toastConfig_.position);
                                $state.go("tab.home", {goLogin: true});
                            } catch (e) {
                                $ionicPopup.alert({
                                    title: "提醒",
                                    template: '登录已超时，请重新登录',
                                    okText: '知道了',
                                    okType: 'button-panda'
                                }).then(function () {
                                    $state.go("tab.home", {goLogin: true});
                                });
                            }
                        } else {
                            if (toastConfig_.show) {
                                try {
                                    $cordovaToast.show(data.message || '服务器内部未知错误', toastConfig_.length, toastConfig_.position);
                                } catch (e) {
                                    $ionicPopup.alert({
                                        title: "错误",
                                        template: data.message || '服务器内部未知错误',
                                        okText: '知道了', // String (default: 'OK'). The text of the OK button.
                                        okType: 'button-panda'
                                    });
                                }
                            }
                            _q.reject({
                                errorMessage: data.message || '服务器内部未知错误',
                                data: data
                            });
                        }

                        UserInfoService.resetSessionTimeOut();

                    }, function (error) {
                        closeIonicLoading(loadingConfig_);

                        if (toastConfig_.show) {
                            try {
                                $cordovaToast.show(error || '请求出现错误了', toastConfig_.length, toastConfig_.position);
                            } catch (e) {
                                $ionicPopup.alert({
                                    title: "错误",
                                    template: error || '请求出现错误了',
                                    okText: '知道了',
                                    okType: 'button-panda'
                                });
                            }
                        }

                        UserInfoService.resetSessionTimeOut();
                        _q.reject({error: error || '请求出现错误了'});

                    });
            };

            var nativeHttpRequest = function (_q, url, data, httpConfig_, loadingConfig_, toastConfig_) {
                showIonicLoading(loadingConfig_);
                $http.post(url, data, httpConfig_)
                    .success(function (data, status, headers, config) {

                        closeIonicLoading(loadingConfig_);

                        _q.notify();
                        if (data && data.code == "00") {

                            _q.resolve(data);

                        } else if(data && data.code == "02") { //版本升级
                            $cordovaToast.show(data.message || '当前版本过低，请升级！', toastConfig_.length, toastConfig_.position);
                            _q.reject();
                        } else if (data && data.code == "TIMEOUT") {
                            UserInfoService.removeUserInfo();

                            try {
                                $cordovaToast.show('登录已超时，请重新登录', toastConfig_.length, toastConfig_.position);
                                $state.go("tab.home", {goLogin: true});
                            } catch (e) {
                                $ionicPopup.alert({
                                    title: "提醒",
                                    template: '登录已超时，请重新登录',
                                    okText: '知道了',
                                    okType: 'button-panda'
                                }).then(function () {
                                    $state.go("tab.home", {goLogin: true});
                                });
                            }

                        } else {
                            if (toastConfig_.show) {
                                try {
                                    $cordovaToast.show(data.message || '服务器内部未知错误', toastConfig_.length, toastConfig_.position);
                                } catch (e) {
                                    $ionicPopup.alert({
                                        title: "错误",
                                        template: data.message || '服务器内部未知错误',
                                        okText: '知道了', // String (default: 'OK'). The text of the OK button.
                                        okType: 'button-panda'
                                    });
                                }
                            }
                            _q.reject({
                                errorMessage: data.message || '服务器内部未知错误',
                                status: status,
                                headers: headers,
                                config: config,
                                data: data
                            });
                        }

                        UserInfoService.resetSessionTimeOut();

                    })
                    .error(function (error, status, headers, config) {

                        closeIonicLoading(loadingConfig_);

                        _q.notify();

                        if (toastConfig_.show) {
                            try {
                                $cordovaToast.show(error || '请求出现错误了', toastConfig_.length, toastConfig_.position);
                            } catch (e) {
                                $ionicPopup.alert({
                                    title: "错误",
                                    template: error || '请求出现错误了',
                                    okText: '知道了',
                                    okType: 'button-panda'
                                });
                            }
                        }

                        UserInfoService.resetSessionTimeOut();

                        _q.reject({error: error || '请求出现错误了', status: status, headers: headers, config: config});
                    });
            }

            //返回promise对象
            return function (url, data, httpConfig, loadingConfig, toastConfig) {
                //_httpConfig.headers.appversion = VersionRandomSerialNO.substring(5);
                var httpConfig_ = angular.extend({}, _httpConfig, httpConfig || {});
                var loadingConfig_ = angular.extend({}, _loadingConfig, loadingConfig || {});
                var toastConfig_ = angular.extend({}, _toastConfig, toastConfig || {});

                var _q = $q.defer();

                if ($rootScope.isOnline) {

                    if (httpConfig_.sha1Channel) {
                        data = angular.extend({}, SHA1ChannelCryptograph(), data);
                    }

                    data = angular.extend({}, {tokenId: UserInfoService.getUserTokenId()}, data);
                    //特性信息
                    data = angular.extend({}, $rootScope.appFeatures, data);

                    //默认全部走加密 需要配置isNotUA=true
                    if (DevelopmentModeConstants.debug) {
                        if (angular.isObject(url)) {
                            nativeHttpRequest(_q, url.url, data, httpConfig_, loadingConfig_, toastConfig_);
                        } else {
                            //angularjs 访问
                            nativeHttpRequest(_q, url, data, httpConfig_, loadingConfig_, toastConfig_);
                        }
                    } else {
                        if (angular.isObject(url)) {
                            if (url.isNotUA) {
                                nativeHttpRequest(_q, url.url, data, httpConfig_, loadingConfig_, toastConfig_);
                            } else {
                                SSLUnilateralismAuthenticationHttpRequest(_q, url.url, data, httpConfig_, loadingConfig_, toastConfig_);
                            }
                        } else {
                            //当url是非Object对象时，若AES值为false走非https请求头
                            if(!DevelopmentModeConstants.aes){
                                nativeHttpRequest(_q, url, data, httpConfig_, loadingConfig_, toastConfig_);
                            }else{
                              SSLUnilateralismAuthenticationHttpRequest(_q, url, data, httpConfig_, loadingConfig_, toastConfig_);
                            }
                        }
                    }

                } else {
                    try {
                        $cordovaToast.show('无网络连接', toastConfig_.length, toastConfig_.position);
                    } catch (e) {
                        $ionicPopup.alert({
                            title: "提醒",
                            template: '无网络连接',
                            okText: '知道了',
                            okType: 'button-panda'
                        });
                    }
                    _q.reject({error: '无网络连接'});

                }

                return _q.promise;
            };

        }]);
